package com.mindoo.bookmarktools.internal;

import lotus.domino.Database;
import lotus.domino.NotesException;
import lotus.domino.Session;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IStartup;

import com.ibm.notes.java.api.data.NotesDatabaseData;
import com.ibm.notes.java.api.util.NotesSessionJob;
import com.ibm.notes.java.ui.NotesUIWorkspace;
import com.ibm.rcp.jface.launcher.LauncherManager;
import com.ibm.rcp.ui.launcher.UrlLauncherContributionItem;
import com.ibm.workplace.noteswc.bookmarks.NotesBookmark;

public class StartupHandler implements IStartup {

	@Override
	public void earlyStartup() {
		//the MenuManager for the open list's context menu has no id; so it's not possible to
		//declare the new context menu actions by using an extension point. Fortunately, there
		//is a public method get a handler on the manager, so we can register an IMenuListener
		final MenuManager mgr=LauncherManager.getInstance().getContextMenuManager();
		mgr.addMenuListener(new IMenuListener() {

			private IAction createLocateOnWorkspaceAction(UrlLauncherContributionItem launcherItem, NotesBookmark nBookmark) {
				LocateOnWorkspaceAction action=new LocateOnWorkspaceAction(launcherItem, nBookmark);
				action.setText("Locate on workspace");
				return action;
			}
			
			private IAction createOpenBookmarkWorkspaceAction(UrlLauncherContributionItem launcherItem, NotesBookmark nBookmark) {
				OpenAsWorkspaceAction action=new OpenAsWorkspaceAction(launcherItem, nBookmark);
				action.setText("Open as workspace");
				return action;
			}

			@Override
			public void menuAboutToShow(IMenuManager paramIMenuManager) {
				if (paramIMenuManager instanceof MenuManager) {
					MenuManager menuMgr=(MenuManager) paramIMenuManager;
					Menu swtMenu=menuMgr.getMenu();
					Object data=swtMenu.getData("com.ibm.rcp.ui.LauncherManager.currentBookmarkItem");
					NotesBookmark nBookmark = null;
					if (data instanceof NotesBookmark) {
						//Check if this is a Notes bookmark, not something that the Notes platform contributes (e.g. Domino Designer launch action)
						nBookmark=(NotesBookmark)data;
					}

					data=swtMenu.getData("com.ibm.rcp.ui.LauncherManager.currentLauncherItem");
					UrlLauncherContributionItem launcherItem = null;
					if (data instanceof UrlLauncherContributionItem) {
						launcherItem=(UrlLauncherContributionItem) data;
					}
					
					if (nBookmark!=null) {
						IAction locateAction=createLocateOnWorkspaceAction(launcherItem, nBookmark);
						IAction openBookmarkWorkspaceAction=createOpenBookmarkWorkspaceAction(launcherItem, nBookmark);
						
						mgr.add(locateAction);
						mgr.add(openBookmarkWorkspaceAction);
					}
				}
			}
		});
		
	}

	/**
	 * This action focuses the clicked database on the workspace.
	 * 
	 * @author Karsten Lehmann
	 */
	private static class LocateOnWorkspaceAction extends Action {
		private UrlLauncherContributionItem m_launcherItem;
		private NotesBookmark m_nBookmark;
		
		public LocateOnWorkspaceAction(UrlLauncherContributionItem launcherItem, NotesBookmark nBookmark) {
			m_launcherItem=launcherItem;
			m_nBookmark=nBookmark;
		}
		
		@Override
		public void run() {
			NotesSessionJob job=new NotesSessionJob("Locate database on workspace") {
				@Override
				protected IStatus runInNotesThread(Session session, IProgressMonitor progress) throws NotesException {
					final String url=m_nBookmark.getURL();

					if (url!=null) {
						progress.beginTask("Resolving database", 3);
						progress.setTaskName("Resolving database");
						NotesUIWorkspace ws=new NotesUIWorkspace();
						NotesDatabaseData dbData=new NotesDatabaseData(url);
						Database db;
						try {
							db=dbData.open(session);
						}
						catch (NotesException e) {
							return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "This function only works if the database physically exists and is located on a server or locally within the data directory", e);
						}
						progress.worked(2);
						progress.setTaskName("Locating database on workspace");
						
						//make sure the workspace is visible:
						ws.openUrl("notes:///ClientBookmark?OpenWorkspace");
						
						ws.addDatabase(new NotesDatabaseData(db.getServer(), db.getFilePath()));

						return Status.OK_STATUS;
					}
					else
						return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "The Notes bookmark is not a database");
				}
			};
			job.setUser(true);
			job.schedule();
		}
	}
	
	/**
	 * This action uses a hidden feature of Lotus Notes to show a bookmark folder as workspace
	 * 
	 * @author Karsten Lehmann
	 */
	private static class OpenAsWorkspaceAction extends Action {
		private UrlLauncherContributionItem m_launcherItem;
		private NotesBookmark m_nBookmark;
		
		public OpenAsWorkspaceAction(UrlLauncherContributionItem launcherItem, NotesBookmark nBookmark) {
			m_launcherItem=launcherItem;
			m_nBookmark=nBookmark;
		}
		
		@Override
		public void run() {
			int parentFolderId=m_nBookmark.getParentFolderId();
			int folderId=m_nBookmark.getFolderId();
			int openFolder=folderId!=0 ? folderId : parentFolderId;
			//open the URL to display the bookmark folder as workspace
			Program.launch("notes:///ClientBookmark?OpenWorkspace&id=" + openFolder);
		}
	}
	
}
